package gomod

import (
	"fmt"
	"net/http"
	"path/filepath"
	"encoding/json"
	"onlinePaySplitGoSDK/models"
	"onlinePaySplitGoSDK/utils"
	
	
)

func OrderRequest(w http.ResponseWriter, r *http.Request) {
	//接口地址
	 apiurl := "https://apis.5upay.com/onlinePay/order"
	 //pfx私钥
	 privatepfxpath, pfxerr := filepath.Abs("package/client.pfx")
	 if pfxerr != nil {  
	    fmt.Println("pfxerr:", pfxerr)  
	 } 
	 pfxpwd := "123456"
     //cer公钥
	 publiccerpath, cererr := filepath.Abs("package/payease.cer")
	 if cererr != nil {  
	    fmt.Println("cererr:", cererr)  
	 }
	 //生成16位随机数
     var aesKeyValue string  = utils.GetRandomString(16)

	 //获取前端参数
	 r.ParseForm()
	 var payorder models.PayOrder
	 payorder.MerchantId = utils.IsNull(r.Form["merchantId"])
	 payorder.OrderAmount = utils.IsNull(r.Form["orderAmount"])
	 payorder.OrderCurrency = utils.IsNull(r.Form["orderCurrency"])
	 payorder.RequestId = utils.IsNull(r.Form["requestId"])
	 payorder.PartnerId = utils.IsNull(r.Form["partnerId"])
	 payorder.NotifyUrl = utils.IsNull(r.Form["notifyUrl"])
	 payorder.CallbackUrl = utils.IsNull(r.Form["callbackUrl"])
	 payorder.Remark = utils.IsNull(r.Form["remark"])
	 payorder.PaymentModeCode = utils.IsNull(r.Form["paymentModeCode"])
	 payorder.AuthCode = utils.IsNull(r.Form["authCode"])
	 payorder.OpenId = utils.IsNull(r.Form["openId"])
	 payorder.AppId = utils.IsNull(r.Form["appId"])
	 payorder.SplitMark = utils.IsNull(r.Form["splitMark"])
	 payorder.ClientIp = utils.IsNull(r.Form["clientIp"])
	 payorder.Timeout = utils.IsNull(r.Form["timeout"])
	 payorder.MerchantUserId = utils.IsNull(r.Form["merchantUserId"])
	 payorder.BindCardId = utils.IsNull(r.Form["bindCardId"])
	 payorder.ReportSerialNo = utils.IsNull(r.Form["reportSerialNo"])
	 payorder.LimitAccType = utils.IsNull(r.Form["limitAccType"])
	 payorder.SubsidyMark = utils.IsNull(r.Form["subsidyMark"])
	 payorder.SubsidyAmount = utils.IsNull(r.Form["subsidyAmount"])
	 payorder.NeedOpenLink = utils.IsNull(r.Form["needOpenLink"])

	 //身份信息
	 var payer models.Payer
	 payer.Name = utils.IsNull(r.Form["payerName"])
     payer.IdType = utils.IsNull(r.Form["idType"])
     payer.IdNum = utils.IsNull(r.Form["idNum"])
     payer.BankCardNum = utils.IsNull(r.Form["bankCardNum"])
     payer.PhoneNum = utils.IsNull(r.Form["phoneNum"])
     payer.Email = utils.IsNull(r.Form["email"])
	 payorder.Payer=payer
	
	 
	 //商品信息
	 var pds [] models.ProductDetail
	 err := json.Unmarshal([]byte(utils.IsNull(r.Form["productDetail"])), &pds)  
	 if err != nil {  
	    fmt.Println("jsonerror:", err)  
	 } 
	 payorder.ProductDetails = pds
    


	 //把订单对象数据转JSON字符串
	 orderdata, err := json.Marshal(payorder)  
	if err != nil {  
	    fmt.Printf(err.Error())  
	    return  
	}


	//排序
	sortdata, oerr := utils.GetMapSortJson(string(orderdata))
	if oerr != nil {
        fmt.Println("oerr:", oerr)
    }
	fmt.Println("排序：",sortdata)
	
	//hamc
	hm := utils.Sha1(sortdata)
	hmac,err := utils.Sign(hm,privatepfxpath,pfxpwd)
	if err != nil {
        fmt.Println("err:", err)
    } 
	payorder.Hmac = hmac
	fmt.Println("hmac：",hmac)	

	orderjson, err := json.Marshal(payorder)  
	if err != nil {  
	    fmt.Printf(err.Error())  
	    return  
	}
	//将byte转换成string进行打印
	fmt.Println("提交订单信息：",string(orderjson))

	//加密的encryptKey秘钥
	encryptKey,enerr := utils.PublicEncrypt(aesKeyValue, publiccerpath)
	if enerr != nil {
        fmt.Println("err:", enerr)
    } 

	// fmt.Println("encryptKey：",encryptKey)

	//返回体 data
	aesdata,aeserro := utils.AesECBEncrypt(string(orderjson), aesKeyValue)
	if aeserro != nil {
        fmt.Println("err:", aeserro)
    } 
	// fmt.Println("aesdata：",aesdata)

	//提交请求
	response, rperr := utils.Post(apiurl,encryptKey,payorder.MerchantId,payorder.RequestId,payorder.PartnerId,aesdata,privatepfxpath,pfxpwd)
	if rperr != nil {
        fmt.Println("rperr:", rperr)
    } 

	//返回
	var orderbacks models.PayOrderBacks
    json.Unmarshal([]byte(response), &orderbacks)
    rhmac := orderbacks.Hmac

    if 	rhmac != ""{
    	sortdeldata, oerr := utils.GetDelJson(string(response),"hmac")
		if oerr != nil {
	        fmt.Println("oerr:", oerr)
	    } 
		signhm := utils.Sha1(sortdeldata)
    	verr := utils.VerifySign(signhm ,publiccerpath,rhmac)
    	if verr == nil {
    		status := orderbacks.Status
    		if status == "REDIRECT"{
    			if orderbacks.JsString != ""{
    				fmt.Fprintf(w, "返回JSAPI字符串内容\n"+ orderbacks.JsString)
    				
    			}else{
					//重定向地址收银台
					http.Redirect(w,r,orderbacks.RedirectUrl,http.StatusFound)
    			}
				
    		}else if status == "SUCCESS"{
    			//返回扫码Base64不为空
    			if orderbacks.ScanCode != ""{
    				//展示二维码
					fmt.Fprintf(w, "返回base64编码\n"+ orderbacks.ScanCode)
					
    			}else if orderbacks.AppParams != ""{//返回APP字符串不为空
    				fmt.Fprintf(w, "返回APP字符串内容\n"+ orderbacks.AppParams)
    				
    			}else if orderbacks.JsString != ""{//小程序返回JSAPI字符串不为空
    				fmt.Fprintf(w, "返回JSAPI字符串内容\n"+ orderbacks.JsString)
    				
    			}else{
    				fmt.Fprintf(w, "成功返回内容\n"+ response)
    			}
    		}else{
    			fmt.Println("失败:", status)
    			fmt.Fprintf(w,"失败:"+status)
    		}
    	}else{
	    	fmt.Println("verr:", verr)
	    	fmt.Fprintf(w,"验签异常!")
    	}
    }else{
    	fmt.Println("返回数据:", response)
    	fmt.Fprintf(w, response)
		
    }
}



